//
// This class converts end-of-line characters among the Unix, DOS, and
// Mac conventions.
//

import java.io.*;

public class EolConverter {

    public static final int UNIX = 0;
    public static final int DOS = 1;
    public static final int MAC = 2;

    public String getFormatString(int format) {
        String rv = null;
        switch( format ) {
        case UNIX:
            rv = "\n";
            break;
        case DOS:
            rv = "\r\n";
            break;
        case MAC:
            rv = "\r";
            break;
        default:
            rv = "\n";
            break;
        }
        return rv;
    }

    public static String getPrefixString(int format) {
        String rv = null;
        switch( format ) {
        case UNIX:
            rv = new String("lf");
            break;
        case DOS:
            rv = new String("crlf");
            break;
        case MAC:
            rv = new String("cr");
            break;
        default:
            rv = new String("lf");
            break;
        }
        return rv;
    }

    EolConverter() {
        this(System.in, System.out, UNIX);
    }

    EolConverter(int format) {
        this(System.in, System.out, format);
    }

    EolConverter(InputStream in, OutputStream out) {
        this(in, out, UNIX);
    }

    // In-place converstion of "fileName" to "format".
    EolConverter(String fileName, int format)
        throws java.io.FileNotFoundException, java.io.IOException
    {
        // Convert fileName to an InputStream and a temporary
        // OutputStream.  Once the conversion is completed, the
        // temporary file will be copied over the original.
        InputStream in;
        OutputStream out;
        if( fileName.equals("-") ) {
            in = System.in;
            out = System.out;
        } else {
            _origFile = new File(fileName);
            in  = new FileInputStream(new File(fileName));
            _tmpFile = File.createTempFile(getPrefixString(format) + "-", null);
            out = new FileOutputStream(_tmpFile);
        }

        setIn(in);
        setOut(out);
        setFormat(format);
    }

    EolConverter(InputStream in, OutputStream out, int format) {
        setIn(in);
        setOut(out);
        setFormat(format);
    }

    public void setIn(InputStream in) {
        _in = new LineNumberReader(new InputStreamReader(in));
    }

    public void setOut(OutputStream out) {
        _out = new PrintWriter(out);
    }

    public int getFormat() {
        return _format;
    }

    public void setFormat(int format) {
        _format = format;
    }

    void convert() throws IOException {
        for( ;; ) {
            String line;
            String formatString = getFormatString(_format);
            if( (line = _in.readLine()) == null ) {
                break;
            }
            _out.print(line.toString() + formatString);
        }

        _out.flush();

        // Don't try to copy the temporary file back over the original
        // file if the user requested stdin to be converted.
        if( (_tmpFile != null) && (_origFile != null) ) {

            _in.close();
            _in = null;

            _out.close();
            _out = null;

            CopyCarefully();
        }
    }

    private void CopyCarefully() {
        try {
            // Windows version of Java will not let you rename "on top
            // of" another file.  So, first remove "foo.crlf-bak" if
            // it exists.  Then, move the original to "foo.crlf-bak".
            // Then, move the converted file to "foo".  Then, delete
            // "foo.crlf-bak".
            File bakFile = new File(_origFile.toString()
                                    + "." + getPrefixString(_format) + "-bak");
            if( bakFile.exists() ) {
                if( !bakFile.delete() ) {
                    throw new FileOpFailedException("Error: Unable to delete "
                                                    + "current backup \""
                                                    + bakFile + "\".  "
                                                    + "Conversion aborted.");
                }
            }

            if( !_origFile.renameTo(bakFile) ) {
                throw new FileOpFailedException("Error: Unable to backup \""
                                                + _origFile + "\".  "
                                                + "Conversion aborted.");
            }

            if( !_tmpFile.renameTo(_origFile) ) {
                throw new FileOpFailedException("Error: Unable to save "
                                                + "conversion for \""
                                                + _origFile + "\".  "
                                                + "Conversion aborted.");
            }

            if( !bakFile.delete() ) {
                // This is not enough of an error to throw an exception.
                System.err.println("Warning: Unable to remove backup \""
                                   + _origFile + "\".  Conversion was "
                                   + "successful though.");
            }

        } catch( FileOpFailedException e ) {
            System.err.println(e.getLocalizedMessage());
        }
    }


    //
    // Private data
    //
    private LineNumberReader _in;
    private PrintWriter _out;
    private int _format;
    private File _origFile;
    private File _tmpFile;
}
